/*
 * Copyright (C) 2018 The Harbby Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.github.harbby.gadtry.aop;

import com.github.harbby.gadtry.aop.model.MethodInfo;
import com.github.harbby.gadtry.aop.model.Pointcut;
import com.github.harbby.gadtry.aop.v1.FilterBuilder;
import com.github.harbby.gadtry.function.Consumer;
import com.github.harbby.gadtry.function.Function;
import com.github.harbby.gadtry.function.Runnable;

import static java.util.Objects.requireNonNull;

public interface Binder
{
    /**
     * Define a pointcut
     *
     * @param pointName cut name
     * @param location cut location
     * @return next set CutMode
     */
    @Deprecated
    public PointBuilder bind(String pointName, String location);

    /**
     * Define a pointcut
     *
     * @param pointName cut name
     * @return LocationOrClass
     */
    public FilterBuilder bind(String pointName);

    public static class PointBuilder
            implements CutMode<PointBuilder>
    {
        private final Pointcut pointcut;

        public PointBuilder(Pointcut pointcut)
        {
            this.pointcut = pointcut;
        }

        @Override
        public PointBuilder around(Consumer<ProxyContext> aroundHandler)
        {
            requireNonNull(aroundHandler, "aroundHandler is null");
            pointcut.setAround(aroundHandler);
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder around(Function<ProxyContext, Object> aroundHandler)
        {
            requireNonNull(aroundHandler, "aroundHandler is null");
            pointcut.setAround(aroundHandler);
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder before(Runnable runnable)
        {
            pointcut.setBefore(info -> runnable.apply());
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder before(Consumer<MethodInfo> runnable)
        {
            pointcut.setBefore(runnable);
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder afterReturning(Runnable runnable)
        {
            pointcut.setAfterReturning(info -> runnable.apply());
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder afterReturning(Consumer<MethodInfo> runnable)
        {
            pointcut.setAfterReturning(runnable);
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder after(Runnable runnable)
        {
            pointcut.setAfter(info -> runnable.apply());
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder after(Consumer<MethodInfo> runnable)
        {
            pointcut.setAfter(runnable);
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder afterThrowing(Runnable runnable)
        {
            pointcut.setAfterThrowing(info -> runnable.apply());
            return new PointBuilder(pointcut);
        }

        @Override
        public PointBuilder afterThrowing(Consumer<MethodInfo> runnable)
        {
            pointcut.setAfterThrowing(runnable);
            return new PointBuilder(pointcut);
        }
    }
}
